from enum import Enum


class PantherLogType(str, Enum):
    AWS_ALB = "AWS.ALB"
    AWS_AuroraMySQLAudit = "AWS.AuroraMySQLAudit"
    AWS_CloudTrail = "AWS.CloudTrail"
    AWS_CloudTrailDigest = "AWS.CloudTrailDigest"
    AWS_CloudTrailInsight = "AWS.CloudTrailInsight"
    AWS_CloudWatchEvents = "AWS.CloudWatchEvents"
    AWS_Config = "AWS.Config"
    AWS_GuardDuty = "AWS.GuardDuty"
    AWS_S3ServerAccess = "AWS.S3ServerAccess"
    AWS_SecurityFindingFormat = "AWS.SecurityFindingFormat"
    AWS_TransitGatewayFlow = "AWS.TransitGatewayFlow"
    AWS_VPCDns = "AWS.VPCDns"
    AWS_VPCFlow = "AWS.VPCFlow"
    AWS_WAFWebACL = "AWS.WAFWebACL"
    AlphaSOC_Alert = "AlphaSOC.Alert"
    Amazon_EKS_Audit = "Amazon.EKS.Audit"
    Amazon_EKS_Authenticator = "Amazon.EKS.Authenticator"
    Anomali_Indicator = "Anomali.Indicator"
    Apache_AccessCombined = "Apache.AccessCombined"
    Apache_AccessCommon = "Apache.AccessCommon"
    AppOmni_Alerts = "AppOmni.Alerts"
    AppOmni_Events = "AppOmni.Events"
    AppOmni_Policy = "AppOmni.Policy"
    Asana_Audit = "Asana.Audit"
    Atlassian_Audit = "Atlassian.Audit"
    Auth0_Events = "Auth0.Events"
    Azure_Audit = "Azure.Audit"
    Azure_DefenderAlerts = "Azure.DefenderAlerts"
    Azure_MonitorActivity = "Azure.MonitorActivity"
    Bitwarden_Events = "Bitwarden.Events"
    Box_Event = "Box.Event"
    CarbonBlack_AlertV2 = "CarbonBlack.AlertV2"
    CarbonBlack_Audit = "CarbonBlack.Audit"
    CarbonBlack_EndpointEvent = "CarbonBlack.EndpointEvent"
    CarbonBlack_WatchlistHit = "CarbonBlack.WatchlistHit"
    CiscoUmbrella_CloudFirewall = "CiscoUmbrella.CloudFirewall"
    CiscoUmbrella_DNS = "CiscoUmbrella.DNS"
    CiscoUmbrella_IP = "CiscoUmbrella.IP"
    CiscoUmbrella_Proxy = "CiscoUmbrella.Proxy"
    Cloudflare_Audit = "Cloudflare.Audit"
    Cloudflare_Firewall = "Cloudflare.Firewall"
    Cloudflare_HttpRequest = "Cloudflare.HttpRequest"
    Cloudflare_Spectrum = "Cloudflare.Spectrum"
    Cloudflare_ZeroTrust_RData = "Cloudflare.ZeroTrust.RData"
    Crowdstrike_AIDMaster = "Crowdstrike.AIDMaster"
    Crowdstrike_ActivityAudit = "Crowdstrike.ActivityAudit"
    Crowdstrike_AppInfo = "Crowdstrike.AppInfo"
    Crowdstrike_CriticalFile = "Crowdstrike.CriticalFile"
    Crowdstrike_DNSRequest = "Crowdstrike.DNSRequest"
    Crowdstrike_DetectionSummary = "Crowdstrike.DetectionSummary"
    Crowdstrike_FDREvent = "Crowdstrike.FDREvent"
    Crowdstrike_GroupIdentity = "Crowdstrike.GroupIdentity"
    Crowdstrike_ManagedAssets = "Crowdstrike.ManagedAssets"
    Crowdstrike_NetworkConnect = "Crowdstrike.NetworkConnect"
    Crowdstrike_NetworkListen = "Crowdstrike.NetworkListen"
    Crowdstrike_NotManagedAssets = "Crowdstrike.NotManagedAssets"
    Crowdstrike_ProcessRollup2 = "Crowdstrike.ProcessRollup2"
    Crowdstrike_ProcessRollup2Stats = "Crowdstrike.ProcessRollup2Stats"
    Crowdstrike_SyntheticProcessRollup2 = "Crowdstrike.SyntheticProcessRollup2"
    Crowdstrike_Unknown = "Crowdstrike.Unknown"
    Crowdstrike_UserIdentity = "Crowdstrike.UserIdentity"
    Crowdstrike_UserInfo = "Crowdstrike.UserInfo"
    Crowdstrike_UserLogonLogoff = "Crowdstrike.UserLogonLogoff"
    Docker_Events = "Docker.Events"
    Dropbox_TeamEvent = "Dropbox.TeamEvent"
    Duo_Administrator = "Duo.Administrator"
    Duo_Authentication = "Duo.Authentication"
    Duo_OfflineEnrollment = "Duo.OfflineEnrollment"
    Duo_Telephony = "Duo.Telephony"
    Envoy_Access = "Envoy.Access"
    Fastly_Access = "Fastly.Access"
    Fluentd_Syslog3164 = "Fluentd.Syslog3164"
    Fluentd_Syslog5424 = "Fluentd.Syslog5424"
    GCP_AuditLog = "GCP.AuditLog"
    GCP_HTTPLoadBalancer = "GCP.HTTPLoadBalancer"
    GSuite_ActivityEvent = "GSuite.ActivityEvent"
    GSuite_DirectoryUsers = "GSuite.DirectoryUsers"
    GSuite_Reports = "GSuite.Reports"
    GitHub_Audit = "GitHub.Audit"
    GitHub_Webhook = "GitHub.Webhook"
    GitLab_API = "GitLab.API"
    GitLab_Audit = "GitLab.Audit"
    GitLab_Exceptions = "GitLab.Exceptions"
    GitLab_Git = "GitLab.Git"
    GitLab_Integrations = "GitLab.Integrations"
    GitLab_Production = "GitLab.Production"
    Gravitational_TeleportAudit = "Gravitational.TeleportAudit"
    GreyNoise_Noise = "GreyNoise.Noise"
    GreyNoise_RIOT = "GreyNoise.RIOT"
    Heroku_Runtime = "Heroku.Runtime"
    IPInfo_ASNCIDR = "IPInfo.ASNCIDR"
    IPInfo_ASNRanges = "IPInfo.ASNRanges"
    IPInfo_LocationCIDR = "IPInfo.LocationCIDR"
    IPInfo_LocationRanges = "IPInfo.LocationRanges"
    IPInfo_PrivacyCIDR = "IPInfo.PrivacyCIDR"
    IPInfo_PrivacyRanges = "IPInfo.PrivacyRanges"
    Jamfpro_ComplianceReporter = "Jamfpro.ComplianceReporter"
    Jamfpro_Login = "Jamfpro.Login"
    Juniper_Access = "Juniper.Access"
    Juniper_Audit = "Juniper.Audit"
    Juniper_Firewall = "Juniper.Firewall"
    Juniper_MWS = "Juniper.MWS"
    Juniper_Postgres = "Juniper.Postgres"
    Juniper_Security = "Juniper.Security"
    Lacework_AgentManagement = "Lacework.AgentManagement"
    Lacework_AlertDetails = "Lacework.AlertDetails"
    Lacework_AllFiles = "Lacework.AllFiles"
    Lacework_Applications = "Lacework.Applications"
    Lacework_ChangeFiles = "Lacework.ChangeFiles"
    Lacework_CloudCompliance = "Lacework.CloudCompliance"
    Lacework_CloudConfiguration = "Lacework.CloudConfiguration"
    Lacework_Cmdline = "Lacework.Cmdline"
    Lacework_Connections = "Lacework.Connections"
    Lacework_ContainerSummary = "Lacework.ContainerSummary"
    Lacework_ContainerVulnDetails = "Lacework.ContainerVulnDetails"
    Lacework_DNSQuery = "Lacework.DNSQuery"
    Lacework_Events = "Lacework.Events"
    Lacework_HostVulnDetails = "Lacework.HostVulnDetails"
    Lacework_Image = "Lacework.Image"
    Lacework_Interfaces = "Lacework.Interfaces"
    Lacework_InternalIPA = "Lacework.InternalIPA"
    Lacework_MachineDetails = "Lacework.MachineDetails"
    Lacework_MachineSummary = "Lacework.MachineSummary"
    Lacework_NewHashes = "Lacework.NewHashes"
    Lacework_Package = "Lacework.Package"
    Lacework_PodSummary = "Lacework.PodSummary"
    Lacework_ProcessSummary = "Lacework.ProcessSummary"
    Lacework_UserDetails = "Lacework.UserDetails"
    Lacework_UserLogin = "Lacework.UserLogin"
    Linux_Auditd = "Linux.Auditd"
    Microsoft365_Audit_AzureActiveDirectory = "Microsoft365.Audit.AzureActiveDirectory"
    Microsoft365_Audit_Exchange = "Microsoft365.Audit.Exchange"
    Microsoft365_Audit_General = "Microsoft365.Audit.General"
    Microsoft365_Audit_SharePoint = "Microsoft365.Audit.SharePoint"
    Microsoft365_DLP_All = "Microsoft365.DLP.All"
    MicrosoftGraph_SecurityAlert = "MicrosoftGraph.SecurityAlert"
    MongoDB_OrganizationEvent = "MongoDB.OrganizationEvent"
    MongoDB_ProjectEvent = "MongoDB.ProjectEvent"
    Netskope_Audit = "Netskope.Audit"
    Nginx_Access = "Nginx.Access"
    Nginx_Error = "Nginx.Error"
    Notion_AuditLogs = "Notion.AuditLogs"
    OCSF_AccountChange = "OCSF.AccountChange"
    OCSF_ApiActivity = "OCSF.ApiActivity"
    OCSF_ApplicationLifecycle = "OCSF.ApplicationLifecycle"
    OCSF_Authentication = "OCSF.Authentication"
    OCSF_AuthorizeSession = "OCSF.AuthorizeSession"
    OCSF_BaseEvent = "OCSF.BaseEvent"
    OCSF_ComplianceFinding = "OCSF.ComplianceFinding"
    OCSF_ConfigState = "OCSF.ConfigState"
    OCSF_DatastoreActivity = "OCSF.DatastoreActivity"
    OCSF_DetectionFinding = "OCSF.DetectionFinding"
    OCSF_DeviceConfigStateChange = "OCSF.DeviceConfigStateChange"
    OCSF_DhcpActivity = "OCSF.DhcpActivity"
    OCSF_DnsActivity = "OCSF.DnsActivity"
    OCSF_EmailActivity = "OCSF.EmailActivity"
    OCSF_EmailFileActivity = "OCSF.EmailFileActivity"
    OCSF_EmailUrlActivity = "OCSF.EmailUrlActivity"
    OCSF_EntityManagement = "OCSF.EntityManagement"
    OCSF_FileActivity = "OCSF.FileActivity"
    OCSF_FileHosting = "OCSF.FileHosting"
    OCSF_FtpActivity = "OCSF.FtpActivity"
    OCSF_GroupManagement = "OCSF.GroupManagement"
    OCSF_HttpActivity = "OCSF.HttpActivity"
    OCSF_IncidentFinding = "OCSF.IncidentFinding"
    OCSF_InventoryInfo = "OCSF.InventoryInfo"
    OCSF_KernelActivity = "OCSF.KernelActivity"
    OCSF_KernelExtension = "OCSF.KernelExtension"
    OCSF_MemoryActivity = "OCSF.MemoryActivity"
    OCSF_ModuleActivity = "OCSF.ModuleActivity"
    OCSF_NetworkActivity = "OCSF.NetworkActivity"
    OCSF_NetworkFileActivity = "OCSF.NetworkFileActivity"
    OCSF_NtpActivity = "OCSF.NtpActivity"
    OCSF_PatchState = "OCSF.PatchState"
    OCSF_ProcessActivity = "OCSF.ProcessActivity"
    OCSF_RdpActivity = "OCSF.RdpActivity"
    OCSF_ScanActivity = "OCSF.ScanActivity"
    OCSF_ScheduledJobActivity = "OCSF.ScheduledJobActivity"
    OCSF_SecurityFinding = "OCSF.SecurityFinding"
    OCSF_SmbActivity = "OCSF.SmbActivity"
    OCSF_SshActivity = "OCSF.SshActivity"
    OCSF_UserAccess = "OCSF.UserAccess"
    OCSF_UserInventory = "OCSF.UserInventory"
    OCSF_VulnerabilityFinding = "OCSF.VulnerabilityFinding"
    OCSF_WebResourceAccessActivity = "OCSF.WebResourceAccessActivity"
    OCSF_WebResourcesActivity = "OCSF.WebResourcesActivity"
    OCSF_WinPrefetchInfo = "OCSF.WinPrefetchInfo"
    OCSF_WinRegistryKeyActivity = "OCSF.WinRegistryKeyActivity"
    OCSF_WinRegistryKeyInfo = "OCSF.WinRegistryKeyInfo"
    OCSF_WinRegistryValueActivity = "OCSF.WinRegistryValueActivity"
    OCSF_WinRegistryValueInfo = "OCSF.WinRegistryValueInfo"
    OCSF_WinResourceActivity = "OCSF.WinResourceActivity"
    OCSF_WinprefetchInfo = "OCSF.WinprefetchInfo"
    OCSF_WinregistryKeyActivity = "OCSF.WinregistryKeyActivity"
    OCSF_WinregistryKeyInfo = "OCSF.WinregistryKeyInfo"
    OCSF_WinregistryValueActivity = "OCSF.WinregistryValueActivity"
    OCSF_WinregistryValueInfo = "OCSF.WinregistryValueInfo"
    OCSF_WinresourceActivity = "OCSF.WinresourceActivity"
    OSSEC_EventInfo = "OSSEC.EventInfo"
    Okta_Devices = "Okta.Devices"
    Okta_SystemLog = "Okta.SystemLog"
    Okta_Users = "Okta.Users"
    OneLogin_Events = "OneLogin.Events"
    OnePassword_AuditEvent = "OnePassword.AuditEvent"
    OnePassword_ItemUsage = "OnePassword.ItemUsage"
    OnePassword_SignInAttempt = "OnePassword.SignInAttempt"
    Osquery_Batch = "Osquery.Batch"
    Osquery_Differential = "Osquery.Differential"
    Osquery_Snapshot = "Osquery.Snapshot"
    Osquery_Status = "Osquery.Status"
    Panther_Audit = "Panther.Audit"
    Proofpoint_Event = "Proofpoint.Event"
    PushSecurity_Activity = "PushSecurity.Activity"
    PushSecurity_AttackDetection = "PushSecurity.AttackDetection"
    PushSecurity_Controls = "PushSecurity.Controls"
    PushSecurity_Entities = "PushSecurity.Entities"
    Salesforce_Login = "Salesforce.Login"
    Salesforce_LoginAs = "Salesforce.LoginAs"
    Salesforce_Logout = "Salesforce.Logout"
    Salesforce_URI = "Salesforce.URI"
    SentinelOne_Activity = "SentinelOne.Activity"
    SentinelOne_DeepVisibility = "SentinelOne.DeepVisibility"
    SentinelOne_DeepVisibilityV2 = "SentinelOne.DeepVisibilityV2"
    Slack_AccessLogs = "Slack.AccessLogs"
    Slack_AuditLogs = "Slack.AuditLogs"
    Slack_IntegrationLogs = "Slack.IntegrationLogs"
    Snyk_GroupAudit = "Snyk.GroupAudit"
    Snyk_OrgAudit = "Snyk.OrgAudit"
    Sophos_Central = "Sophos.Central"
    Suricata_Alert = "Suricata.Alert"
    Suricata_Anomaly = "Suricata.Anomaly"
    Suricata_DHCP = "Suricata.DHCP"
    Suricata_DNS = "Suricata.DNS"
    Suricata_FileInfo = "Suricata.FileInfo"
    Suricata_Flow = "Suricata.Flow"
    Suricata_HTTP = "Suricata.HTTP"
    Suricata_SSH = "Suricata.SSH"
    Suricata_TLS = "Suricata.TLS"
    Sysdig_Audit = "Sysdig.Audit"
    Syslog_RFC3164 = "Syslog.RFC3164"
    Syslog_RFC5424 = "Syslog.RFC5424"
    Tailscale_Audit = "Tailscale.Audit"
    Tailscale_Network = "Tailscale.Network"
    Tenable_Vulnerability = "Tenable.Vulnerability"
    Tines_Audit = "Tines.Audit"
    Tor_ExitNode = "Tor.ExitNode"
    TrailDiscover_CloudTrail = "TrailDiscover.CloudTrail"
    Windows_EventLogs = "Windows.EventLogs"
    Workday_Activity = "Workday.Activity"
    Workday_SignOnAttempt = "Workday.SignOnAttempt"
    Zeek_CaptureLoss = "Zeek.CaptureLoss"
    Zeek_Conn = "Zeek.Conn"
    Zeek_DHCP = "Zeek.DHCP"
    Zeek_DNS = "Zeek.DNS"
    Zeek_DPD = "Zeek.DPD"
    Zeek_Files = "Zeek.Files"
    Zeek_HTTP = "Zeek.HTTP"
    Zeek_NTP = "Zeek.NTP"
    Zeek_Notice = "Zeek.Notice"
    Zeek_OCSP = "Zeek.OCSP"
    Zeek_Reporter = "Zeek.Reporter"
    Zeek_SIP = "Zeek.SIP"
    Zeek_Software = "Zeek.Software"
    Zeek_Ssh = "Zeek.Ssh"
    Zeek_Ssl = "Zeek.Ssl"
    Zeek_Stats = "Zeek.Stats"
    Zeek_Tunnel = "Zeek.Tunnel"
    Zeek_Weird = "Zeek.Weird"
    Zeek_X509 = "Zeek.X509"
    Zendesk_Audit = "Zendesk.Audit"
    Zoom_Activity = "Zoom.Activity"
    Zoom_Operation = "Zoom.Operation"

    def __str__(self) -> str:
        """Returns a string representation of the class' value."""
        return self.value

    @staticmethod
    def get_attribute_name(value: str) -> str:
        """Returns the attribute name of the class' value."""
        for name, member in PantherLogType.__members__.items():
            if member.value == value:
                return name
        raise AttributeError(f"LogType has no attribute with value {value}")



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

box_access_granted_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Regular Event",
        ExpectedResult=False,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "DELETE",
        },
    ),
    PantherRuleTest(
        Name="Access Granted",
        ExpectedResult=True,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "ACCESS_GRANTED",
            "source": {
                "id": "12345678",
                "type": "user",
                "login": "user@example",
                "name": "Bob Cat",
            },
        },
    ),
]


class BoxAccessGranted(PantherRule):
    RuleID = "Box.Access.Granted-prototype"
    DisplayName = "Box Access Granted"
    LogTypes = [PantherLogType.Box_Event]
    Tags = ["Box"]
    Severity = PantherSeverity.Low
    Description = "A user granted access to their box account to Box technical support from account settings.\n"
    Reference = "https://support.box.com/hc/en-us/articles/7039943421715-Enabling-and-Disabling-Access-for-Box-Support"
    Runbook = "Investigate whether the user purposefully granted access to their account.\n"
    SummaryAttributes = ["p_any_ip_addresses"]
    Tests = box_access_granted_tests

    def rule(self, event):
        return event.get("event_type") == "ACCESS_GRANTED"

    def title(self, event):
        return f"User [{deep_get(event, 'created_by', 'name', default='<UNKNOWN_USER>')}] granted access to their account"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

panther_saml_modified_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="SAML config modified",
        ExpectedResult=True,
        Log={
            "actionName": "UPDATE_SAML_SETTINGS",
            "actionParams": {},
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {
                    "email": "homer@springfield.gov",
                    "emailVerified": True,
                    "roleId": "111111",
                },
                "id": "111111",
                "name": "Homer Simpson",
                "type": "USER",
            },
            "errors": None,
            "p_log_type": "Panther.Audit",
        },
    ),
    PantherRuleTest(
        Name="SAML config viewed",
        ExpectedResult=False,
        Log={
            "actionName": "GET_SAML_SETTINGS",
            "actionParams": {},
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {
                    "email": "homer@springfield.gov",
                    "emailVerified": True,
                    "roleId": "111111",
                },
                "id": "111111",
                "name": "Homer Simpson",
                "type": "USER",
            },
            "errors": None,
            "p_log_type": "Panther.Audit",
        },
    ),
]


class PantherSAMLModified(PantherRule):
    RuleID = "Panther.SAML.Modified-prototype"
    DisplayName = "Panther SAML configuration has been modified"
    LogTypes = [PantherLogType.Panther_Audit]
    Severity = PantherSeverity.High
    Tags = ["DataModel", "Defense Evasion:Impair Defenses"]
    Reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    Description = "An Admin has modified Panther's SAML configuration."
    Runbook = "Ensure this change was approved and appropriate."
    Reference = "https://docs.panther.com/system-configuration/saml"
    SummaryAttributes = ["p_any_ip_addresses", "p_any_usernames"]
    Tests = panther_saml_modified_tests

    def rule(self, event):
        return (
            event.get("actionName") == "UPDATE_SAML_SETTINGS"
            and event.get("actionResult") == "SUCCEEDED"
        )

    def title(self, event):
        return f"Panther SAML config has been modified by {event.udm('actor_user')}"

    def alert_context(self, event):
        return {"user": event.udm("actor_user"), "ip": event.udm("source_ip")}



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import github_alert_context
from pypanther.log_types import PantherLogType

github_public_repository_created_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Public Repo Created",
        ExpectedResult=True,
        Log={
            "_document_id": "abCD",
            "action": "repo.create",
            "actor": "example-actor",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-12-11 22:40:20.268",
            "created_at": "2022-12-11 22:40:20.268",
            "org": "example-io",
            "repo": "example-io/oops",
            "visibility": "public",
        },
    ),
    PantherRuleTest(
        Name="Private Repo Created",
        ExpectedResult=False,
        Log={
            "_document_id": "abCD",
            "action": "repo.create",
            "actor": "example-actor",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-12-11 22:40:20.268",
            "created_at": "2022-12-11 22:40:20.268",
            "org": "example-io",
            "repo": "example-io/oops",
            "visibility": "private",
        },
    ),
]


class GithubPublicRepositoryCreated(PantherRule):
    Description = "A public Github repository was created."
    DisplayName = "Github Public Repository Created"
    Runbook = (
        "Confirm this github repository was intended to be created as 'public' versus 'private'."
    )
    Reference = "https://docs.github.com/en/get-started/quickstart/create-a-repo"
    Severity = PantherSeverity.Medium
    Tags = ["Github Repository", "Public", "Repository Created"]
    LogTypes = [PantherLogType.GitHub_Audit]
    RuleID = "Github.Public.Repository.Created-prototype"
    SummaryAttributes = ["actor", "repository", "visibility"]
    Tests = github_public_repository_created_tests
    # def dedup(event):
    #  (Optional) Return a string which will be used to deduplicate similar alerts.
    # return ''

    def rule(self, event):
        # Return True if a public repository was created
        return event.get("action", "") == "repo.create" and event.get("visibility", "") == "public"

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this method
        # will act as deduplication string.
        return f"Repository [{event.get('repo', '<UNKNOWN_REPO>')}] created with public status by Github user [{event.get('actor')}]."

    def alert_context(self, event):
        #  (Optional) Return a dictionary with additional data to be included in the alert
        # sent to the SNS/SQS/Webhook destination
        return github_alert_context(event)



import json
from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

duo_admin_lockout_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Admin lockout- invalid json",
        ExpectedResult=True,
        Log={
            "action": "admin_lockout",
            "description": '"message": "Admin temporarily locked out due to too many passcode attempts."',
            "isotimestamp": "2022-12-14 21:02:03",
            "timestamp": "2022-12-14 21:02:03",
            "username": "Homer Simpson",
        },
    ),
    PantherRuleTest(
        Name="Admin lockout- valid json",
        ExpectedResult=True,
        Log={
            "action": "admin_lockout",
            "description": '{"message": "Admin temporarily locked out due to too many passcode attempts."}',
            "isotimestamp": "2022-12-14 21:02:03",
            "timestamp": "2022-12-14 21:02:03",
            "username": "Homer Simpson",
        },
    ),
    PantherRuleTest(
        Name="Bypass Create",
        ExpectedResult=False,
        Log={
            "action": "bypass_create",
            "description": '{"bypass": "", "count": 1, "valid_secs": 3600, "auto_generated": true, "remaining_uses": 1, "user_id": "D12345", "bypass_code_ids": ["A12345"]}',
            "isotimestamp": "2022-12-14 21:17:39",
            "object": "target@example.io",
            "timestamp": "2022-12-14 21:17:39",
            "username": "Homer Simpson",
        },
    ),
]


class DuoAdminLockout(PantherRule):
    Description = "Alert when a duo administrator is locked out of their account."
    DisplayName = "Duo Admin Lockout"
    Reference = "https://duo.com/docs/adminapi"
    Severity = PantherSeverity.Medium
    LogTypes = [PantherLogType.Duo_Administrator]
    RuleID = "Duo.Admin.Lockout-prototype"
    Tests = duo_admin_lockout_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        return event.get("action", "") == "admin_lockout"

    def title(self, event):
        # If no 'dedup' function is defined, the return value
        # of this method will act as deduplication string.
        try:
            desc = json.loads(event.get("description", {}))
            message = desc.get("message", "<NO_MESSAGE_FOUND>")[:-1]
        except ValueError:
            message = "Invalid Json"
        return f"Duo Admin [{event.get('username', '<NO_USER_FOUND>')}] is locked out. Reason: [{message}]."



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.log_types import PantherLogType

aws_user_login_profile_modified_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="ChangeOwnPassword",
        ExpectedResult=False,
        Log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "1234",
            "eventName": "UpdateLoginProfile",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-09-15 13:45:24",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "987654321",
            "requestParameters": {"passwordResetRequired": False, "userName": "alice"},
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ABC1234",
                "accountId": "987654321",
                "arn": "arn:aws:sts::98765432:assumed-role/IAM/alice",
                "principalId": "ABCDE:alice",
                "sessionContext": {
                    "attributes": {
                        "creationDate": "2022-09-15T13:36:47Z",
                        "mfaAuthenticated": "true",
                    },
                    "sessionIssuer": {
                        "accountId": "987654321",
                        "arn": "arn:aws:iam::9876432:role/IAM",
                        "principalId": "1234ABC",
                        "type": "Role",
                        "userName": "IAM",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    PantherRuleTest(
        Name="User changed password for other",
        ExpectedResult=True,
        Log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "1234",
            "eventName": "UpdateLoginProfile",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-09-15 13:45:24",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "987654321",
            "requestParameters": {"passwordResetRequired": False, "userName": "bob"},
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ABC1234",
                "accountId": "987654321",
                "arn": "arn:aws:sts::98765432:assumed-role/IAM/alice",
                "principalId": "ABCDE:alice",
                "sessionContext": {
                    "attributes": {
                        "creationDate": "2022-09-15T13:36:47Z",
                        "mfaAuthenticated": "true",
                    },
                    "sessionIssuer": {
                        "accountId": "987654321",
                        "arn": "arn:aws:iam::9876432:role/IAM",
                        "principalId": "1234ABC",
                        "type": "Role",
                        "userName": "IAM",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    PantherRuleTest(
        Name="User changed password for other reset required",
        ExpectedResult=False,
        Log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "1234",
            "eventName": "UpdateLoginProfile",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-09-15 13:45:24",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "987654321",
            "requestParameters": {"passwordResetRequired": True, "userName": "bob"},
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ABC1234",
                "accountId": "987654321",
                "arn": "arn:aws:sts::98765432:assumed-role/IAM/alice",
                "principalId": "ABCDE:alice",
                "sessionContext": {
                    "attributes": {
                        "creationDate": "2022-09-15T13:36:47Z",
                        "mfaAuthenticated": "true",
                    },
                    "sessionIssuer": {
                        "accountId": "987654321",
                        "arn": "arn:aws:iam::9876432:role/IAM",
                        "principalId": "1234ABC",
                        "type": "Role",
                        "userName": "IAM",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


class AWSUserLoginProfileModified(PantherRule):
    Description = "An attacker with iam:UpdateLoginProfile permission on other users can change the password used to login to the AWS console. May be legitimate account administration."
    DisplayName = "AWS User Login Profile Modified"
    Reports = {"MITRE ATT&CK": ["TA0003:T1098", "TA0005:T1108", "TA0005:T1550", "TA0008:T1550"]}
    Reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_examples_aws_my-sec-creds-self-manage-pass-accesskeys-ssh.html"
    Severity = PantherSeverity.High
    LogTypes = [PantherLogType.AWS_CloudTrail]
    RuleID = "AWS.User.Login.Profile.Modified-prototype"
    Tests = aws_user_login_profile_modified_tests

    def rule(self, event):
        return (
            event.get("eventSource", "") == "iam.amazonaws.com"
            and event.get("eventName", "") == "UpdateLoginProfile"
            and (not deep_get(event, "requestParameters", "passwordResetRequired", default=False))
            and (
                not deep_get(event, "userIdentity", "arn", default="").endswith(
                    f"/{deep_get(event, 'requestParameters', 'userName', default='')}"
                )
            )
        )

    def title(self, event):
        return f"User [{deep_get(event, 'userIdentity', 'arn').split('/')[-1]}] changed the password for [{deep_get(event, 'requestParameters', 'userName')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get, okta_alert_context
from pypanther.log_types import PantherLogType

okta_api_key_revoked_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="API Key Revoked",
        ExpectedResult=True,
        Log={
            "uuid": "2a992f80-d1ad-4f62-900e-8c68bb72a21b",
            "published": "2021-01-08 21:28:34.875",
            "eventType": "system.api_token.revoke",
            "version": "0",
            "severity": "INFO",
            "legacyEventType": "api.token.revoke",
            "displayMessage": "Revoke API token",
            "actor": {
                "alternateId": "user@example.com",
                "displayName": "Test User",
                "id": "00u3q14ei6KUOm4Xi2p4",
                "type": "User",
            },
            "outcome": {"result": "SUCCESS"},
            "request": {},
            "debugContext": {},
            "target": [
                {
                    "id": "00Tpki36zlWjhjQ1u2p4",
                    "type": "Token",
                    "alternateId": "unknown",
                    "displayName": "test_key",
                    "details": None,
                }
            ],
        },
    )
]


class OktaAPIKeyRevoked(PantherRule):
    RuleID = "Okta.APIKeyRevoked-prototype"
    DisplayName = "Okta API Key Revoked"
    LogTypes = [PantherLogType.Okta_SystemLog]
    Tags = ["Identity & Access Management", "Okta"]
    Severity = PantherSeverity.Info
    Description = "A user has revoked an API Key in Okta"
    Reference = "https://help.okta.com/en/prod/Content/Topics/Security/API.htm"
    Runbook = "Validate this action was authorized."
    SummaryAttributes = ["eventType", "severity", "displayMessage", "p_any_ip_addresses"]
    Tests = okta_api_key_revoked_tests

    def rule(self, event):
        return (
            event.get("eventType", None) == "system.api_token.revoke"
            and deep_get(event, "outcome", "result") == "SUCCESS"
        )

    def title(self, event):
        target = event.get("target", [{}])
        key_name = (
            target[0].get("displayName", "MISSING DISPLAY NAME") if target else "MISSING TARGET"
        )
        return f"{deep_get(event, 'actor', 'displayName')} <{deep_get(event, 'actor', 'alternateId')}>revoked API key - <{key_name}>"

    def alert_context(self, event):
        return okta_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

g_suite_advanced_protection_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Advanced Protection Enabled",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "user_accounts"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.com"},
            "type": "titanium_change",
            "name": "titanium_enroll",
        },
    ),
    PantherRuleTest(
        Name="Advanced Protection Disabled",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "user_accounts"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.com"},
            "type": "titanium_change",
            "name": "titanium_unenroll",
        },
    ),
]


class GSuiteAdvancedProtection(PantherRule):
    RuleID = "GSuite.AdvancedProtection-prototype"
    DisplayName = "GSuite User Advanced Protection Change"
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite", "Defense Evasion:Impair Defenses"]
    Reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    Severity = PantherSeverity.Low
    Description = "A user disabled advanced protection for themselves.\n"
    Reference = "https://support.google.com/a/answer/9378686?hl=en&sjid=864417124752637253-EU"
    Runbook = "Have the user re-enable Google Advanced Protection\n"
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_advanced_protection_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "user_accounts":
            return False
        return bool(event.get("name") == "titanium_unenroll")

    def title(self, event):
        return f"Advanced protection was disabled for user [{deep_get(event, 'actor', 'email', default='<UNKNOWN_EMAIL>')}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

g_suite_device_unlock_failure_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Normal Mobile Event",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "DEVICE_SYNC_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io"},
        },
    ),
    PantherRuleTest(
        Name="Small Number of Failed Logins",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "FAILED_PASSWORD_ATTEMPTS_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io", "FAILED_PASSWD_ATTEMPTS": 2},
        },
    ),
    PantherRuleTest(
        Name="Multiple Failed Login Attempts with int Type",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "FAILED_PASSWORD_ATTEMPTS_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io", "FAILED_PASSWD_ATTEMPTS": 100},
        },
    ),
    PantherRuleTest(
        Name="Multiple Failed Login Attempts with String Type",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "FAILED_PASSWORD_ATTEMPTS_EVENT",
            "parameters": {
                "USER_EMAIL": "homer.simpson@example.io",
                "FAILED_PASSWD_ATTEMPTS": "100",
            },
        },
    ),
]


class GSuiteDeviceUnlockFailure(PantherRule):
    RuleID = "GSuite.DeviceUnlockFailure-prototype"
    DisplayName = "GSuite User Device Unlock Failures"
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite", "Credential Access:Brute Force"]
    Reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    Severity = PantherSeverity.Medium
    Description = "Someone failed to unlock a user's device multiple times in quick succession.\n"
    Reference = "https://support.google.com/a/answer/6350074?hl=en"
    Runbook = "Verify that these unlock attempts came from the user, and not a malicious actor which has acquired the user's device.\n"
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_device_unlock_failure_tests
    MAX_UNLOCK_ATTEMPTS = 10

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "mobile":
            return False
        if event.get("name") == "FAILED_PASSWORD_ATTEMPTS_EVENT":
            attempts = deep_get(event, "parameters", "FAILED_PASSWD_ATTEMPTS")
            return int(attempts if attempts else 0) > self.MAX_UNLOCK_ATTEMPTS
        return False

    def title(self, event):
        return f"User [{deep_get(event, 'actor', 'email', default='<UNKNOWN_USER>')}]'s device had multiple failed unlock attempts"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

cisco_umbrella_dns_suspicious_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Suspicious Domain",
        ExpectedResult=True,
        Log={
            "action": "Allow",
            "internalIp": "136.24.229.58",
            "externalIp": "136.24.229.58",
            "timestamp": "2020-05-21 19:20:25.000",
            "responseCode": "NOERROR",
            "domain": "cron.photoscape.ch.",
        },
    ),
    PantherRuleTest(
        Name="Safe Domain",
        ExpectedResult=False,
        Log={
            "action": "Allowed",
            "internalIp": "136.24.229.58",
            "externalIp": "136.24.229.58",
            "timestamp": "2020-05-21 19:20:25.000",
            "responseCode": "NOERROR",
            "domain": "google.com.",
        },
    ),
]


class CiscoUmbrellaDNSSuspicious(PantherRule):
    RuleID = "CiscoUmbrella.DNS.Suspicious-prototype"
    DisplayName = "Cisco Umbrella Suspicious Domains"
    Enabled = False
    DedupPeriodMinutes = 480
    LogTypes = [PantherLogType.CiscoUmbrella_DNS]
    Tags = ["DNS", "Configuration Required"]
    Reference = "https://umbrella.cisco.com/blog/abcs-of-dns"
    Severity = PantherSeverity.Low
    Description = "Monitor suspicious or known malicious domains"
    Runbook = "Inspect the domain and check the host for other indicators of compromise"
    SummaryAttributes = ["action", "internalIp", "externalIp", "domain", "responseCode"]
    Tests = cisco_umbrella_dns_suspicious_tests
    DOMAINS_TO_MONITOR = {"photoscape.ch"}  # Sample malware domain

    def rule(self, event):
        return any((domain in event.get("domain") for domain in self.DOMAINS_TO_MONITOR))

    def title(self, event):
        return "Suspicious lookup to domain " + event.get("domain", "<UNKNOWN_DOMAIN>")



from typing import Iterable, Set, Type

from pypanther.base import PantherRule

__REGISTRY: Set[Type[PantherRule]] = set()


def register(rule: Type[PantherRule] | Iterable[Type[PantherRule]]):
    if isinstance(rule, type) and issubclass(rule, PantherRule):
        register_rule(rule)
        return
    try:
        for r in iter(rule):
            register_rule(r)
        return
    except TypeError:
        pass

    raise ValueError(f"rule must be a PantherRule or an iterable of PantherRule not {rule}")


def register_rule(rule: Type[PantherRule]):
    if not isinstance(rule, type) and issubclass(rule, PantherRule):
        raise ValueError(f"rule must be a PantherRule subclass not {rule}")

    rule.validate()
    __REGISTRY.add(rule)


def registered_rules() -> Set[Type[PantherRule]]:
    return __REGISTRY



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

teleport_saml_created_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="SAML Auth Connector modified",
        ExpectedResult=True,
        Log={
            "cluster_name": "teleport.example.com",
            "code": "T8200I",
            "ei": 0,
            "event": "saml.created",
            "name": "okta",
            "time": "2023-09-19 18:00:00",
            "uid": "88888888-4444-4444-4444-222222222222",
            "user": "max.mustermann@zumbeispiel.example",
        },
    )
]


class TeleportSAMLCreated(PantherRule):
    RuleID = "Teleport.SAMLCreated-prototype"
    DisplayName = "A SAML Connector was created or modified"
    LogTypes = [PantherLogType.Gravitational_TeleportAudit]
    Tags = ["Teleport"]
    Severity = PantherSeverity.High
    Description = "A SAML connector was created or modified"
    Reports = {"MITRE ATT&CK": ["TA0042:T1585"]}
    Reference = "https://goteleport.com/docs/management/admin/"
    Runbook = "When a SAML connector is modified, it can potentially change the trust model of the Teleport Cluster. Validate that these changes were expected and correct.\n"
    SummaryAttributes = ["event", "code", "user", "name"]
    Tests = teleport_saml_created_tests

    def rule(self, event):
        return event.get("event") == "saml.created"

    def title(self, event):
        return f"A SAML connector was created or updated by User [{event.get('user', '<UNKNOWN_USER>')}] on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"



from difflib import SequenceMatcher
from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType


class CiscoUmbrellaDNSFuzzyMatching(PantherRule):
    RuleID = "CiscoUmbrella.DNS.FuzzyMatching-prototype"
    DisplayName = "Cisco Umbrella Domain Name Fuzzy Matching"
    Enabled = False
    DedupPeriodMinutes = 15
    LogTypes = [PantherLogType.CiscoUmbrella_DNS]
    Tags = ["Configuration Required", "DNS"]
    Reference = "https://umbrella.cisco.com/blog/abcs-of-dns"
    Severity = PantherSeverity.Medium
    Description = "Identify lookups to suspicious domains that could indicate a phishing attack."
    Runbook = "Validate if your organization owns the domain, otherwise investigate the host that made the domain resolution.\n"
    DOMAIN = ""  # The domain to monitor for phishing, for example "google.com"
    # List all of your known-good domains here
    ALLOW_SET = {}
    SIMILARITY_RATIO = 0.7

    def rule(self, event):
        # Domains coming through umbrella end with a dot, such as google.com.
        domain = ".".join(event.get("domain").rstrip(".").split(".")[-2:]).lower()
        return (
            domain not in self.ALLOW_SET
            and SequenceMatcher(None, self.DOMAIN, domain).ratio() >= self.SIMILARITY_RATIO
        )

    def title(self, event):
        return f"Suspicious DNS resolution to {event.get('domain')}"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get, okta_alert_context
from pypanther.log_types import PantherLogType

okta_support_reset_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Support Reset Credential",
        ExpectedResult=True,
        Log={
            "uuid": "12343",
            "published": "2021-11-29 18:56:40.014",
            "eventType": "user.account.reset_password",
            "version": "0",
            "severity": "INFO",
            "legacyEventType": "core.user.config.user_status.password_reset",
            "displayMessage": "Fired when the user's Okta password is reset",
            "actor": {
                "alternateId": "system@okta.com",
                "displayName": "system@okta.com",
                "id": "1111111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "ipAddress": "1.1.1.1",
                "userAgent": {"browser": "CHROME", "os": "Mac OS X"},
                "zone": "null",
            },
            "outcome": {"result": "SUCCESS"},
            "target": [
                {
                    "alternateId": "homer@springfield.gov",
                    "displayName": "Homer Simpson",
                    "id": "1111111",
                    "type": "User",
                }
            ],
            "transaction": {"detail": {}, "id": "unknown", "type": "WEB"},
            "p_log_type": "Okta.SystemLog",
        },
    ),
    PantherRuleTest(
        Name="Reset by Company Admin",
        ExpectedResult=False,
        Log={
            "uuid": "2aaaaaaaaaabbbbbbbbbbbbddddddddddd",
            "eventType": "user.account.reset_password",
            "version": "0",
            "severity": "INFO",
            "legacyEventType": "core.user.config.user_status.password_reset",
            "displayMessage": "Fired when the user's Okta password is reset",
            "actor": {
                "alternateId": "marge@springfield.gov",
                "displayName": "Marge Simpson",
                "id": "1111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "postalCode": "80014",
                    "state": "Debated",
                },
                "ipAddress": "1.1.1.1",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.55 Safari/537.36",
                },
                "zone": "null",
            },
            "outcome": {"result": "SUCCESS"},
            "target": [
                {
                    "alternateId": "homer@springfield.gov",
                    "displayName": "Homer Simpson",
                    "id": "1.1.1.1",
                    "type": "User",
                }
            ],
            "transaction": {"detail": {}, "id": "1111", "type": "WEB"},
            "p_log_type": "Okta.SystemLog",
        },
    ),
]


class OktaSupportReset(PantherRule):
    RuleID = "Okta.Support.Reset-prototype"
    DisplayName = "Okta Support Reset Credential"
    LogTypes = [PantherLogType.Okta_SystemLog]
    Tags = [
        "Identity & Access Management",
        "DataModel",
        "Okta",
        "Initial Access:Trusted Relationship",
    ]
    Reports = {"MITRE ATT&CK": ["TA0001:T1199"]}
    Severity = PantherSeverity.High
    Description = "A Password or MFA factor was reset by Okta Support"
    Reference = "https://help.okta.com/en/prod/Content/Topics/Directory/get-support.htm#:~:text=Visit%20the%20Okta%20Help%20Center,1%2D800%2D219%2D0964"
    Runbook = "Contact Admin to ensure this was sanctioned activity"
    DedupPeriodMinutes = 15
    SummaryAttributes = ["eventType", "severity", "p_any_ip_addresses"]
    Tests = okta_support_reset_tests
    OKTA_SUPPORT_RESET_EVENTS = [
        "user.account.reset_password",
        "user.mfa.factor.update",
        "system.mfa.factor.deactivate",
        "user.mfa.attempt_bypass",
    ]

    def rule(self, event):
        if event.get("eventType") not in self.OKTA_SUPPORT_RESET_EVENTS:
            return False
        return (
            deep_get(event, "actor", "alternateId") == "system@okta.com"
            and deep_get(event, "transaction", "id") == "unknown"
            and (deep_get(event, "userAgent", "rawUserAgent") is None)
            and (deep_get(event, "client", "geographicalContext", "country") is None)
        )

    def title(self, event):
        return f"Okta Support Reset Password or MFA for user {event.udm('actor_user')}"

    def alert_context(self, event):
        return okta_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

netskope_netskope_personnel_activity_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="True positive",
        ExpectedResult=True,
        Log={
            "_id": "e5ca619b059fccdd0cfd9398",
            "_insertion_epoch_timestamp": 1702308331,
            "audit_log_event": "Login Successful",
            "count": 1,
            "is_netskope_personnel": True,
            "organization_unit": "",
            "severity_level": 2,
            "supporting_data": {
                "data_type": "user",
                "data_values": ["11.22.33.44", "adminsupport@netskope.com"],
            },
            "timestamp": "2023-12-11 15:25:31.000000000",
            "type": "admin_audit_logs",
            "ur_normalized": "adminsupport@netskope.com",
            "user": "adminsupport@netskope.com",
        },
    ),
    PantherRuleTest(
        Name="True negative",
        ExpectedResult=False,
        Log={
            "_id": "1e589befa3da30132362f32a",
            "_insertion_epoch_timestamp": 1702318213,
            "audit_log_event": "Rest API V2 Call",
            "count": 1,
            "is_netskope_personnel": False,
            "organization_unit": "",
            "severity_level": 2,
            "supporting_data": {
                "data_type": "incidents",
                "data_values": [
                    200,
                    "POST",
                    "/api/v2/incidents/uba/getuci",
                    "trid=ccb898fgrhvdd0v0lebg",
                ],
            },
            "timestamp": "2023-12-11 18:10:13.000000000",
            "type": "admin_audit_logs",
            "ur_normalized": "service-account",
            "user": "service-account",
        },
    ),
]


class NetskopeNetskopePersonnelActivity(PantherRule):
    RuleID = "Netskope.NetskopePersonnelActivity-prototype"
    DisplayName = "Action Performed by Netskope Personnel"
    LogTypes = [PantherLogType.Netskope_Audit]
    Tags = ["Netskope", "Supply Chain Compromise"]
    Reports = {"MITRE ATT&CK": ["TA0001:T1195"]}
    Severity = PantherSeverity.Medium
    Description = "An action was performed by Netskope personnel."
    Runbook = "Action taken by Netskope Personnel.  Validate that this action was authorized."
    Reference = "https://docs.netskope.com/en/netskope-help/admin-console/administration/audit-log/#filters-1"
    Tests = netskope_netskope_personnel_activity_tests

    def rule(self, event):
        if event.get("is_netskope_personnel") is True:
            return True
        return False

    def title(self, event):
        user = event.get("user", "<USER_NOT_FOUND>")
        audit_log_event = event.get("audit_log_event", "<EVENT_NOT_FOUND>")
        return f"Action [{audit_log_event}] performed by Netskope personnel [{user}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import aws_guardduty_context, deep_get
from pypanther.log_types import PantherLogType

aws_guard_duty_low_severity_finding_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Low Sev Finding",
        ExpectedResult=True,
        Log={
            "schemaVersion": "2.0",
            "accountId": "123456789012",
            "region": "us-east-1",
            "partition": "aws",
            "arn": "arn:aws:guardduty:us-west-2:123456789012:detector/111111bbbbbbbbbb5555555551111111/finding/90b82273685661b9318f078d0851fe9a",
            "type": "PrivilegeEscalation:IAMUser/AdministrativePermissions",
            "service": {
                "serviceName": "guardduty",
                "detectorId": "111111bbbbbbbbbb5555555551111111",
                "action": {
                    "actionType": "AWS_API_CALL",
                    "awsApiCallAction": {
                        "api": "PutRolePolicy",
                        "serviceName": "iam.amazonaws.com",
                        "callerType": "Domain",
                        "domainDetails": {"domain": "cloudformation.amazonaws.com"},
                        "affectedResources": {
                            "AWS::IAM::Role": "arn:aws:iam::123456789012:role/IAMRole"
                        },
                    },
                },
                "resourceRole": "TARGET",
                "additionalInfo": {},
                "evidence": None,
                "eventFirstSeen": "2020-02-14T17:59:17Z",
                "eventLastSeen": "2020-02-14T17:59:17Z",
                "archived": False,
                "count": 1,
            },
            "severity": 1,
            "id": "eeb88ab56556eb7771b266670dddee5a",
            "createdAt": "2020-02-14T18:12:22.316Z",
            "updatedAt": "2020-02-14T18:12:22.316Z",
            "title": "Principal AssumedRole:IAMRole attempted to add a policy to themselves that is highly permissive.",
            "description": "Principal AssumedRole:IAMRole attempted to add a highly permissive policy to themselves.",
        },
    ),
    PantherRuleTest(
        Name="Low Sev Finding As Sample Data",
        ExpectedResult=False,
        Log={
            "schemaVersion": "2.0",
            "accountId": "123456789012",
            "region": "us-east-1",
            "partition": "aws",
            "arn": "arn:aws:guardduty:us-west-2:123456789012:detector/111111bbbbbbbbbb5555555551111111/finding/90b82273685661b9318f078d0851fe9a",
            "type": "PrivilegeEscalation:IAMUser/AdministrativePermissions",
            "service": {
                "serviceName": "guardduty",
                "detectorId": "111111bbbbbbbbbb5555555551111111",
                "action": {
                    "actionType": "AWS_API_CALL",
                    "awsApiCallAction": {
                        "api": "PutRolePolicy",
                        "serviceName": "iam.amazonaws.com",
                        "callerType": "Domain",
                        "domainDetails": {"domain": "cloudformation.amazonaws.com"},
                        "affectedResources": {
                            "AWS::IAM::Role": "arn:aws:iam::123456789012:role/IAMRole"
                        },
                    },
                },
                "resourceRole": "TARGET",
                "additionalInfo": {"sample": True},
                "evidence": None,
                "eventFirstSeen": "2020-02-14T17:59:17Z",
                "eventLastSeen": "2020-02-14T17:59:17Z",
                "archived": False,
                "count": 1,
            },
            "severity": 1,
            "id": "eeb88ab56556eb7771b266670dddee5a",
            "createdAt": "2020-02-14T18:12:22.316Z",
            "updatedAt": "2020-02-14T18:12:22.316Z",
            "title": "Principal AssumedRole:IAMRole attempted to add a policy to themselves that is highly permissive.",
            "description": "Principal AssumedRole:IAMRole attempted to add a highly permissive policy to themselves.",
        },
    ),
]


class AWSGuardDutyLowSeverityFinding(PantherRule):
    RuleID = "AWS.GuardDuty.LowSeverityFinding-prototype"
    DisplayName = "AWS GuardDuty Low Severity Finding"
    LogTypes = [PantherLogType.AWS_GuardDuty]
    Tags = ["AWS"]
    Severity = PantherSeverity.Low
    DedupPeriodMinutes = 1440
    Description = "A low-severity GuardDuty finding has been identified.\n"
    Runbook = "Search related logs to understand the root cause of the activity.\n"
    Reference = "https://docs.aws.amazon.com/guardduty/latest/ug/guardduty_findings.html#guardduty_findings-severity"
    SummaryAttributes = [
        "severity",
        "type",
        "title",
        "p_any_domain_names",
        "p_any_aws_arns",
        "p_any_aws_account_ids",
    ]
    Tests = aws_guard_duty_low_severity_finding_tests

    def rule(self, event):
        if deep_get(event, "service", "additionalInfo", "sample"):
            # in case of sample data
            # https://docs.aws.amazon.com/guardduty/latest/ug/sample_findings.html
            return False
        return 0.1 <= float(event.get("severity", 0)) <= 3.9

    def title(self, event):
        return event.get("title")

    def alert_context(self, event):
        return aws_guardduty_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_tailscale_helpers import (
    is_tailscale_admin_console_event,
    tailscale_alert_context,
)
from pypanther.log_types import PantherLogType

tailscale_magic_dns_disabled_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Magic DNS Disabled",
        ExpectedResult=True,
        Log={
            "event": {
                "action": "DISABLE",
                "actor": {
                    "displayName": "Homer Simpson",
                    "id": "uodc9f3CNTRL",
                    "loginName": "homer.simpson@yourcompany.io",
                    "type": "USER",
                },
                "eventGroupID": "017676eb3de31cd31c0be96b965c2970",
                "origin": "ADMIN_CONSOLE",
                "target": {
                    "id": "yoururl.com",
                    "name": "yoururl.com",
                    "property": "MAGIC_DNS",
                    "type": "TAILNET",
                },
            },
            "fields": {"recorded": "2023-07-19 16:10:38.825360311"},
            "p_any_actor_ids": ["uodc9f3CNTRL"],
            "p_any_emails": ["homer.simpson@yourcompany.io"],
            "p_any_usernames": ["homer.simpson"],
            "p_event_time": "2023-07-19 16:10:38.365000",
            "p_log_type": "Tailscale.Audit",
            "p_parse_time": "2023-07-19 16:13:56.849016",
            "p_row_id": "5e197fb53834e39eeab7feb9198904",
            "p_schema_version": 0,
            "p_source_id": "5d65e24a-7ebb-403b-803c-51396e03d201",
            "p_source_label": "Tailscale Audit and Network Logs",
            "p_udm": {},
            "time": "2023-07-19 16:10:38.365000000",
        },
    ),
    PantherRuleTest(
        Name="Other Event",
        ExpectedResult=False,
        Log={
            "event": {
                "action": "CREATE",
                "actor": {
                    "displayName": "Homer Simpson",
                    "id": "uodc9f3CNTRL",
                    "loginName": "homer.simpson@yourcompany.io",
                    "type": "USER",
                },
                "eventGroupID": "9f880e02981e341447958344b7b4071f",
                "new": {},
                "origin": "ADMIN_CONSOLE",
                "target": {"id": "k6r3fm3CNTRL", "name": "API key", "type": "API_KEY"},
            },
            "fields": {"recorded": "2023-07-19 16:11:41.778839718"},
            "p_any_actor_ids": ["uodc9f3CNTRL"],
            "p_any_emails": ["homer.simpson@yourcompany.io"],
            "p_any_usernames": ["homersimpson"],
            "p_event_time": "2023-07-19 16:11:41.601000",
            "p_log_type": "Tailscale.Audit",
            "p_parse_time": "2023-07-19 16:14:56.865276",
            "p_row_id": "02eaf97ec9caaaabff8882ba19ad1d",
            "p_schema_version": 0,
            "p_source_id": "5d65e24a-7ebb-403b-803c-51396e03d201",
            "p_source_label": "Tailscale Audit and Network Logs",
            "p_udm": {},
            "time": "2023-07-19 16:11:41.601000000",
        },
    ),
]


class TailscaleMagicDNSDisabled(PantherRule):
    Description = "A Tailscale User disabled magic dns settings in your organization's tenant."
    DisplayName = "Tailscale Magic DNS Disabled"
    Runbook = "Assess if this was done by the user for a valid business reason. Be vigilant to re-enable this setting as it's in the best security interest for your organization's security posture."
    Reference = "https://tailscale.com/kb/1081/magicdns/"
    Severity = PantherSeverity.High
    LogTypes = [PantherLogType.Tailscale_Audit]
    RuleID = "Tailscale.Magic.DNS.Disabled-prototype"
    Tests = tailscale_magic_dns_disabled_tests

    def rule(self, event):
        action = deep_get(event, "event", "action", default="<NO_ACTION_FOUND>")
        target_property = deep_get(
            event, "event", "target", "property", default="<NO_TARGET_PROPERTY_FOUND>"
        )
        return all(
            [
                action == "DISABLE",
                target_property == "MAGIC_DNS",
                is_tailscale_admin_console_event(event),
            ]
        )

    def title(self, event):
        user = deep_get(event, "event", "actor", "loginName", default="<NO_USER_FOUND>")
        target_id = deep_get(event, "event", "target", "id", default="<NO_TARGET_ID_FOUND>")
        return f"Tailscale user [{user}] disabled Magic DNS for [{target_id}] in your organization’s tenant."

    def alert_context(self, event):
        return tailscale_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import box_parse_additional_details, deep_get
from pypanther.log_types import PantherLogType

box_shield_suspicious_alert_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Regular Event",
        ExpectedResult=False,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "ceo@example",
                "name": "Bob Cat",
            },
            "event_type": "DELETE",
        },
    ),
    PantherRuleTest(
        Name="Suspicious Login Event",
        ExpectedResult=True,
        Log={
            "type": "event",
            "additional_details": '{"shield_alert":{"rule_category":"Suspicious Locations","risk_score":60,"user":{"email":"bob@example"}}}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "bob@example",
                "name": "Bob Cat",
            },
            "event_type": "SHIELD_ALERT",
            "source": {"id": "12345678", "type": "user"},
        },
    ),
    PantherRuleTest(
        Name="Suspicious Session Event",
        ExpectedResult=True,
        Log={
            "type": "event",
            "additional_details": '{"shield_alert":{"rule_category":"Suspicious Sessions","risk_score":70,"alert_summary":{"description":"First time in prior month user connected from ip 1.2.3.4."},"user":{"email":"bob@example"}}}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "bob@example",
                "name": "Bob Cat",
            },
            "event_type": "SHIELD_ALERT",
            "source": {"id": "12345678", "type": "user"},
        },
    ),
    PantherRuleTest(
        Name="Suspicious Session Event - Low Risk",
        ExpectedResult=False,
        Log={
            "type": "event",
            "additional_details": '{"shield_alert":{"rule_category":"Suspicious Sessions","risk_score":10,"alert_summary":{"description":"First time in prior month user connected from ip 1.2.3.4."},"user":{"email":"bob@example"}}}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "bob@example",
                "name": "Bob Cat",
            },
            "event_type": "SHIELD_ALERT",
            "source": {"id": "12345678", "type": "user"},
        },
    ),
]


class BoxShieldSuspiciousAlert(PantherRule):
    RuleID = "Box.Shield.Suspicious.Alert-prototype"
    DisplayName = "Box Shield Suspicious Alert Triggered"
    LogTypes = [PantherLogType.Box_Event]
    Tags = ["Box", "Initial Access:Valid Accounts"]
    Reports = {"MITRE ATT&CK": ["TA0001:T1078"]}
    Severity = PantherSeverity.High
    Description = (
        "A user login event or session event was tagged as medium to high severity by Box Shield.\n"
    )
    Reference = "https://developer.box.com/guides/events/shield-alert-events/"
    Runbook = "Investigate whether this was triggered by an expected user event.\n"
    SummaryAttributes = ["event_type", "ip_address"]
    Tests = box_shield_suspicious_alert_tests
    SUSPICIOUS_EVENT_TYPES = {"Suspicious Locations", "Suspicious Sessions"}

    def rule(self, event):
        if event.get("event_type") != "SHIELD_ALERT":
            return False
        alert_details = box_parse_additional_details(event).get("shield_alert", {})
        if alert_details.get("rule_category", "") in self.SUSPICIOUS_EVENT_TYPES:
            if alert_details.get("risk_score", 0) > 50:
                return True
        return False

    def title(self, event):
        details = box_parse_additional_details(event)
        description = deep_get(details, "shield_alert", "alert_summary", "description", default="")
        if description:
            return description
        return f"Shield medium to high risk, suspicious event alert triggered for user [{deep_get(details, 'shield_alert', 'user', 'email', default='<UNKNOWN_USER>')}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

github_repo_created_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="GitHub - Repo Created",
        ExpectedResult=True,
        Log={
            "actor": "cat",
            "action": "repo.create",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
    PantherRuleTest(
        Name="GitHub - Repo Archived",
        ExpectedResult=False,
        Log={
            "actor": "cat",
            "action": "repo.archived",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
]


class GithubRepoCreated(PantherRule):
    RuleID = "Github.Repo.Created-prototype"
    DisplayName = "GitHub Repository Created"
    LogTypes = [PantherLogType.GitHub_Audit]
    Tags = ["GitHub"]
    Reference = "https://docs.github.com/en/get-started/quickstart/create-a-repo"
    Severity = PantherSeverity.Info
    Description = "Detects when a repository is created."
    Tests = github_repo_created_tests

    def rule(self, event):
        return event.get("action") == "repo.create"

    def title(self, event):
        return f"Repository [{event.get('repo', '<UNKNOWN_REPO>')}] created."



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


class StandardOCSFNetworkActivity(PantherDataModel):
    DataModelID: str = "Standard.OCSF.NetworkActivity"
    DisplayName: str = "OCSF Network Activity"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.OCSF_NetworkActivity]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="destination_ip", Path="$.dst_endpoint.ip"),
        PantherDataModelMapping(Name="destination_port", Path="$.dst_endpoint.port"),
        PantherDataModelMapping(Name="source_ip", Path="$.src_endpoint.ip"),
        PantherDataModelMapping(Name="source_port", Path="$.src_endpoint.port"),
        PantherDataModelMapping(Name="log_status", Path="status_code"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

gcpk8s_pod_using_host_pid_namespace_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="triggers",
        ExpectedResult=True,
        Log={
            "authorizationInfo": [
                {
                    "granted": True,
                    "permission": "io.k8s.core.v1.pods.create",
                    "resource": "core/v1/namespaces/default/pods/nginx-test",
                }
            ],
            "protoPayload": {
                "methodName": "io.k8s.core.v1.pods.create",
                "request": {"spec": {"hostPID": True}},
            },
        },
    ),
    PantherRuleTest(
        Name="ignore",
        ExpectedResult=False,
        Log={
            "authorizationInfo": [
                {
                    "granted": True,
                    "permission": "io.k8s.core.v1.pods.create",
                    "resource": "core/v1/namespaces/default/pods/nginx-test",
                }
            ],
            "protoPayload": {
                "methodName": "io.k8s.core.v1.pods.create",
                "request": {"spec": {"hostPID": False}},
            },
        },
    ),
]


class GCPK8sPodUsingHostPIDNamespace(PantherRule):
    RuleID = "GCP.K8s.Pod.Using.Host.PID.Namespace-prototype"
    DisplayName = "GCP K8s Pod Using Host PID Namespace"
    LogTypes = [PantherLogType.GCP_AuditLog]
    Tags = ["GCP", "Optional"]
    Severity = PantherSeverity.Medium
    Description = "This detection monitors for any pod creation or modification using the host PID namespace. The Host PID namespace enables a pod and its containers to have direct access and share the same view as of the host’s processes. This can offer a powerful escape hatch to the underlying host."
    Runbook = "Investigate a reason of creating a pod using the host PID namespace. Advise that it is discouraged practice. Create ticket if appropriate."
    Reports = {"MITRE ATT&CK": ["TA0004:T1611", "TA0002:T1610"]}
    Reference = "https://medium.com/snowflake/from-logs-to-detection-using-snowflake-and-panther-to-detect-k8s-threats-d72f70a504d7"
    Tests = gcpk8s_pod_using_host_pid_namespace_tests
    METHODS_TO_CHECK = [
        "io.k8s.core.v1.pods.create",
        "io.k8s.core.v1.pods.update",
        "io.k8s.core.v1.pods.patch",
    ]

    def rule(self, event):
        method = deep_get(event, "protoPayload", "methodName")
        request_host_pid = deep_get(event, "protoPayload", "request", "spec", "hostPID")
        response_host_pid = deep_get(event, "protoPayload", "responce", "spec", "hostPID")
        if (
            request_host_pid is True or response_host_pid is True
        ) and method in self.METHODS_TO_CHECK:
            return True
        return False

    def title(self, event):
        actor = deep_get(
            event,
            "protoPayload",
            "authenticationInfo",
            "principalEmail",
            default="<ACTOR_NOT_FOUND>",
        )
        project_id = deep_get(
            event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>"
        )
        return f"[GCP]: [{actor}] created or modified pod using the host PID namespace in project [{project_id}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_auth0_helpers import auth0_alert_context
from pypanther.log_types import PantherLogType

auth0_cic_credential_stuffing_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Auth0 Credential Stuffing Event",
        ExpectedResult=True,
        Log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-23 20:47:51.149000000",
                "description": "Someone behind the IP address ip attempted to login with a leaked password.",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {"jti": "e6343ec1d24a41e6bd43a6be748cac11"},
                            "strategy": "jwt",
                            "user": {
                                "email": "homer.simpson@yourcompany.com",
                                "name": "Homer Simpson",
                                "user_id": "google-oauth2|105261262156475850461",
                            },
                        },
                        "body": {"integration_id": "64bee519-818f-4473-ab08-7c380f28da77"},
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "post",
                        "path": "/api/v2/integrations/installed",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {
                        "body": {"integration_id": "64bee519-818f-4473-ab08-7c380f28da77"},
                        "statusCode": 200,
                    },
                },
                "ip": "12.12.12.12",
                "log_id": "90020230523204756343781000000000000001223372037583230452",
                "type": "pwd_leak",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "google-oauth2|105261262156475850461",
            },
            "log_id": "90020230523204756343781000000000000001223372037583230452",
        },
    ),
    PantherRuleTest(
        Name="Other Event",
        ExpectedResult=False,
        Log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-23 20:47:51.149000000",
                "description": "Install an available integration",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "949869e066205b5076e6df203fdd7b9b",
                                "scopes": [
                                    "create:actions",
                                    "create:actions_log_sessions",
                                    "create:authentication_methods",
                                    "create:client_credentials",
                                    "create:client_grants",
                                    "create:clients",
                                    "create:connections",
                                    "create:custom_domains",
                                    "create:email_provider",
                                    "create:email_templates",
                                    "create:guardian_enrollment_tickets",
                                    "create:integrations",
                                    "create:log_streams",
                                    "create:organization_connections",
                                    "create:organization_invitations",
                                    "create:organization_member_roles",
                                    "create:organization_members",
                                    "create:organizations",
                                    "create:requested_scopes",
                                    "create:resource_servers",
                                    "create:roles",
                                    "create:rules",
                                    "create:shields",
                                    "create:signing_keys",
                                    "create:tenant_invitations",
                                    "create:test_email_dispatch",
                                    "create:users",
                                    "delete:actions",
                                    "delete:anomaly_blocks",
                                    "delete:authentication_methods",
                                    "delete:branding",
                                    "delete:client_credentials",
                                    "delete:client_grants",
                                    "delete:clients",
                                    "delete:connections",
                                    "delete:custom_domains",
                                    "delete:device_credentials",
                                    "delete:email_provider",
                                    "delete:email_templates",
                                    "delete:grants",
                                    "delete:guardian_enrollments",
                                    "delete:integrations",
                                    "delete:log_streams",
                                    "delete:organization_connections",
                                    "delete:organization_invitations",
                                    "delete:organization_member_roles",
                                    "delete:organization_members",
                                    "delete:organizations",
                                    "delete:owners",
                                    "delete:requested_scopes",
                                    "delete:resource_servers",
                                    "delete:roles",
                                    "delete:rules",
                                    "delete:rules_configs",
                                    "delete:shields",
                                    "delete:tenant_invitations",
                                    "delete:tenant_members",
                                    "delete:tenants",
                                    "delete:users",
                                    "read:actions",
                                    "read:anomaly_blocks",
                                    "read:attack_protection",
                                    "read:authentication_methods",
                                    "read:branding",
                                    "read:checks",
                                    "read:client_credentials",
                                    "read:client_grants",
                                    "read:client_keys",
                                    "read:clients",
                                    "read:connections",
                                    "read:custom_domains",
                                    "read:device_credentials",
                                    "read:email_provider",
                                    "read:email_templates",
                                    "read:email_triggers",
                                    "read:entity_counts",
                                    "read:grants",
                                    "read:guardian_factors",
                                    "read:insights",
                                    "read:integrations",
                                    "read:log_streams",
                                    "read:logs",
                                    "read:mfa_policies",
                                    "read:organization_connections",
                                    "read:organization_invitations",
                                    "read:organization_member_roles",
                                    "read:organization_members",
                                    "read:organizations",
                                    "read:prompts",
                                    "read:requested_scopes",
                                    "read:resource_servers",
                                    "read:roles",
                                    "read:rules",
                                    "read:rules_configs",
                                    "read:shields",
                                    "read:signing_keys",
                                    "read:stats",
                                    "read:tenant_invitations",
                                    "read:tenant_members",
                                    "read:tenant_settings",
                                    "read:triggers",
                                    "read:users",
                                    "run:checks",
                                    "update:actions",
                                    "update:attack_protection",
                                    "update:authentication_methods",
                                    "update:branding",
                                    "update:client_credentials",
                                    "update:client_grants",
                                    "update:client_keys",
                                    "update:clients",
                                    "update:connections",
                                    "update:custom_domains",
                                    "update:email_provider",
                                    "update:email_templates",
                                    "update:email_triggers",
                                    "update:guardian_factors",
                                    "update:integrations",
                                    "update:log_streams",
                                    "update:mfa_policies",
                                    "update:organization_connections",
                                    "update:organizations",
                                    "update:prompts",
                                    "update:requested_scopes",
                                    "update:resource_servers",
                                    "update:roles",
                                    "update:rules",
                                    "update:rules_configs",
                                    "update:shields",
                                    "update:signing_keys",
                                    "update:tenant_members",
                                    "update:tenant_settings",
                                    "update:triggers",
                                    "update:users",
                                ],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "user.name@yourcompany.io",
                                "name": "User Name",
                                "user_id": "google-oauth2|105261262156475850461",
                            },
                        },
                        "body": {"AfterAuthentication": False},
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "patch",
                        "path": "/api/v2/risk-assessment/config",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {
                        "body": {
                            "AfterAuthentication": False,
                            "BeforeLoginPrompt": False,
                            "BeforeLoginPromptMonitoring": False,
                        },
                        "statusCode": 200,
                    },
                },
                "ip": "12.12.12.12",
                "log_id": "90020230523204756343781000000000000001223372037583230452",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "google-oauth2|105261262156475850461",
            },
            "log_id": "90020230523204756343781000000000000001223372037583230452",
        },
    ),
]


class Auth0CICCredentialStuffing(PantherRule):
    LogTypes = [PantherLogType.Auth0_Events]
    RuleID = "Auth0.CIC.Credential.Stuffing-prototype"
    DisplayName = "Auth0 CIC Credential Stuffing"
    Description = "Okta has determined that the cross-origin authentication feature in Customer Identity Cloud (CIC) is prone to being targeted by threat actors orchestrating credential-stuffing attacks.  Okta has observed suspicious activity that started on April 15, 2024.  Review tenant logs for unexpected fcoa, scoa, and pwd_leak events."
    Severity = PantherSeverity.High
    Runbook = "If a user password was compromised in a credential stuffing attack, the user's credentials should be rotated immediately out of an abundance of caution."
    Reference = "https://sec.okta.com/articles/2024/05/detecting-cross-origin-authentication-credential-stuffing-attacks"
    Tests = auth0_cic_credential_stuffing_tests
    SUSPICIOUS_EVENT_TYPES = ("scoa", "fcoa", "pwd_leak")

    def rule(self, event):
        return event.deep_get("data", "type") in self.SUSPICIOUS_EVENT_TYPES

    def title(self, event):
        event_type = event.deep_get("data", "type")
        user = event.deep_get(
            "data", "details", "request", "auth", "user", "email", default="<NO_USER_FOUND>"
        )
        p_source_label = event.deep_get("p_source_label", default="<NO_P_SOURCE_LABEL_FOUND>")
        return f"Auth0 User [{user}] had a suspicious [{event_type}] event in your organization's tenant [{p_source_label}]."

    def alert_context(self, event):
        return auth0_alert_context(event)



from typing import List

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.helpers.panther_azuresignin_helpers import actor_user, is_sign_in_event
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType


def get_event_type(event):
    if not is_sign_in_event(event):
        return None

    error_code = deep_get(event, "properties", "status", "errorCode", default=0)
    if error_code == 0:
        return event_type.SUCCESSFUL_LOGIN
    return event_type.FAILED_LOGIN


def get_actor_user(event):
    return actor_user(event)


class StandardAzureAuditSignIn(PantherDataModel):
    DataModelID: str = "Standard.Azure.Audit.SignIn"
    DisplayName: str = "Azure SignIn Logs DataModel"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.Azure_Audit]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Method=get_actor_user),
        PantherDataModelMapping(Name="event_type", Method=get_event_type),
        PantherDataModelMapping(Name="source_ip", Path="$.properties.ipAddress"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

osquery_mac_application_firewall_settings_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="ALF Disabled",
        ExpectedResult=True,
        Log={
            "name": "pack_incident-response_alf",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "logging_enabled": "0",
                "stealth_enabled": "0",
                "firewall_unload": "0",
                "allow_signed_enabled": "0",
                "global_state": "0",
                "logging_option": "0",
                "version": "1.6",
            },
        },
    ),
    PantherRuleTest(
        Name="ALF Enabled",
        ExpectedResult=False,
        Log={
            "name": "pack_incident-response_alf",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "logging_enabled": "1",
                "stealth_enabled": "1",
                "firewall_unload": "0",
                "allow_signed_enabled": "1",
                "global_state": "1",
                "logging_option": "0",
                "version": "1.6",
            },
        },
    ),
]


class OsqueryMacApplicationFirewallSettings(PantherRule):
    RuleID = "Osquery.Mac.ApplicationFirewallSettings-prototype"
    DisplayName = "MacOS ALF is misconfigured"
    LogTypes = [PantherLogType.Osquery_Differential]
    Tags = ["Osquery", "MacOS", "Security Control", "Defense Evasion:Impair Defenses"]
    Reports = {"CIS": ["2.6.3", "2.6.4"], "MITRE ATT&CK": ["TA0005:T1562"]}
    Severity = PantherSeverity.High
    Description = "The application level firewall blocks unwanted network connections made to your computer from other computers on your network.\n"
    Runbook = "Re-enable the firewall manually or with configuration management"
    Reference = "https://support.apple.com/en-us/HT201642"
    SummaryAttributes = ["name", "hostIdentifier", "action"]
    Tests = osquery_mac_application_firewall_settings_tests
    QUERIES = {"pack_incident-response_alf", "pack/mac-cis/ApplicationFirewall"}

    def rule(self, event):
        if event.get("name") not in self.QUERIES:
            return False
        if event.get("action") != "added":
            return False
        # 0 If the firewall is disabled
        # 1 If the firewall is enabled with exceptions
        # 2 If the firewall is configured to block all incoming connections
        # Stealth mode is a best practice to avoid responding to unsolicited probes
        return (
            int(deep_get(event, "columns", "global_state")) == 0
            or int(deep_get(event, "columns", "stealth_enabled")) == 0
        )

    def title(self, event):
        return f"MacOS firewall disabled on [{event.get('hostIdentifier')}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

atlassian_user_logged_in_as_user_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Admin impersonated user successfully",
        ExpectedResult=True,
        Log={
            "attributes": {
                "action": "user_logged_in_as_user",
                "actor": {
                    "email": "example.admin@example.com",
                    "id": "1234567890abcdefghijklmn",
                    "name": "Example Admin",
                },
                "container": [
                    {
                        "attributes": {
                            "siteHostName": "https://example.atlassian.net",
                            "siteName": "example",
                        },
                        "id": "12345678-abcd-9012-efgh-1234567890abcd",
                        "links": {"alt": "https://example.atlassian.net"},
                        "type": "sites",
                    }
                ],
                "context": [
                    {
                        "attributes": {
                            "accountType": "atlassian",
                            "email": "example.user@example.io",
                            "name": "example.user@example.io",
                        },
                        "type": "users",
                    }
                ],
                "time": "2022-12-15T00:35:15.890Z",
            },
            "id": "2508d209-3336-4763-89a0-aceaf1322fcf",
            "message": {"content": "Logged in as example.user@example.io", "format": "simple"},
        },
    ),
    PantherRuleTest(
        Name="user_logged_in_as_user not in log",
        ExpectedResult=False,
        Log={
            "attributes": {
                "action": "user_login",
                "actor": {
                    "email": "example.admin@example.com",
                    "id": "1234567890abcdefghijklmn",
                    "name": "Example Admin",
                },
                "container": [
                    {
                        "attributes": {
                            "siteHostName": "https://example.atlassian.net",
                            "siteName": "example",
                        },
                        "id": "12345678-abcd-9012-efgh-1234567890abcd",
                        "links": {"alt": "https://example.atlassian.net"},
                        "type": "sites",
                    }
                ],
                "context": [
                    {
                        "attributes": {
                            "accountType": "atlassian",
                            "email": "example.user@example.io",
                            "name": "example.user@example.io",
                        },
                        "type": "users",
                    }
                ],
                "time": "2022-12-15T00:35:15.890Z",
            },
            "id": "2508d209-3336-4763-89a0-aceaf1322fcf",
            "message": {"content": "Logged in as example.user@example.io", "format": "simple"},
        },
    ),
]


class AtlassianUserLoggedInAsUser(PantherRule):
    DisplayName = "Atlassian admin impersonated another user"
    RuleID = "Atlassian.User.LoggedInAsUser-prototype"
    Severity = PantherSeverity.High
    LogTypes = [PantherLogType.Atlassian_Audit]
    Tags = ["Atlassian", "User impersonation"]
    Description = "Reports when an Atlassian user logs in (impersonates) another user.\n"
    Runbook = "Validate that the Atlassian admin did log in (impersonate) as another user.\n"
    Reference = "https://support.atlassian.com/user-management/docs/log-in-as-another-user/"
    Tests = atlassian_user_logged_in_as_user_tests

    def rule(self, event):
        return (
            deep_get(event, "attributes", "action", default="<unknown-action>")
            == "user_logged_in_as_user"
        )

    def title(self, event):
        actor = deep_get(event, "attributes", "actor", "email", default="<unknown-email>")
        context = deep_get(event, "attributes", "context", default=[{}])
        impersonated_user = context[0].get("attributes", {}).get("email", "<unknown-email>")
        return f"{actor} logged in as {impersonated_user}."

    def alert_context(self, event):
        return {
            "Timestamp": deep_get(event, "attributes", "time", default="<unknown-time>"),
            "Actor": deep_get(
                event, "attributes", "actor", "email", default="<unknown-actor-email>"
            ),
            "Impersonated user": deep_get(event, "attributes", "context", default=[{}])[0]
            .get("attributes", {})
            .get("email", "<unknown-email>"),
            "Event ID": event.get("id"),
        }



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

github_repo_visibility_change_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="GitHub - Repo Visibility Change",
        ExpectedResult=True,
        Log={
            "actor": "cat",
            "action": "repo.access",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
    PantherRuleTest(
        Name="GitHub - Repo disabled",
        ExpectedResult=False,
        Log={
            "actor": "cat",
            "action": "repo.disable",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
]


class GithubRepoVisibilityChange(PantherRule):
    RuleID = "Github.Repo.VisibilityChange-prototype"
    DisplayName = "GitHub Repository Visibility Change"
    LogTypes = [PantherLogType.GitHub_Audit]
    Tags = ["GitHub", "Exfiltration:Exfiltration Over Web Service"]
    Reports = {"MITRE ATT&CK": ["TA0010:T1567"]}
    Reference = "https://docs.github.com/en/repositories/managing-your-repositorys-settings-and-features/managing-repository-settings/setting-repository-visibility"
    Severity = PantherSeverity.High
    Description = "Detects when an organization repository visibility changes."
    Tests = github_repo_visibility_change_tests

    def rule(self, event):
        return event.get("action") == "repo.access"

    def title(self, event):
        repo_access_link = (
            f"https://github.com/{event.get('repo', '<UNKNOWN_REPO>')}/settings/access"
        )
        return f"Repository [{event.get('repo', '<UNKNOWN_REPO>')}] visibility changed. View current visibility here: {repo_access_link}"



from typing import List

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


def get_event_type(event):
    # currently, only tracking a handful of event types
    if event.get("event_type_id") == 72 and event.get("privilege_name") == "Super user":
        return event_type.ADMIN_ROLE_ASSIGNED
    if event.get("event_type_id") == 6:
        return event_type.FAILED_LOGIN
    if event.get("event_type_id") == 5:
        return event_type.SUCCESSFUL_LOGIN
    if event.get("event_type_id") == 13:
        return event_type.USER_ACCOUNT_CREATED
    return None


class StandardOneLoginEvents(PantherDataModel):
    DataModelID: str = "Standard.OneLogin.Events"
    DisplayName: str = "OneLogin Events"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.OneLogin_Events]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Path="actor_user_name"),
        PantherDataModelMapping(Name="assigned_admin_role", Path="privilege_name"),
        PantherDataModelMapping(Name="event_type", Method=get_event_type),
        PantherDataModelMapping(Name="source_ip", Path="ipaddr"),
        PantherDataModelMapping(Name="user", Path="user_name"),
        PantherDataModelMapping(Name="user_account_id", Path="user_id"),
        PantherDataModelMapping(Name="user_agent", Path="user_agent"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

carbon_black_audit_api_key_created_retrieved_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="API Key Retrieved",
        ExpectedResult=True,
        Log={
            "clientIp": "12.34.56.78",
            "description": "User bob.ross@acme.com retrieved secret for API ID JFDNIPS464 in org 12345",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "verbose": False,
        },
    ),
    PantherRuleTest(
        Name="Admin granted",
        ExpectedResult=True,
        Log={
            "clientIp": "12.34.56.78",
            "description": "Added API ID JFDNIPS464 with name evil-key in org 12345",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "verbose": False,
        },
    ),
    PantherRuleTest(
        Name="Other role granted",
        ExpectedResult=False,
        Log={
            "clientIp": "12.34.56.78",
            "description": "Created grant: psc:cnn:A1234567:BC1234567890 with role Read Only",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "requestUrl": "/access/v2/orgs/A1234567/grants",
            "verbose": False,
        },
    ),
]


class CarbonBlackAuditAPIKeyCreatedRetrieved(PantherRule):
    RuleID = "CarbonBlack.Audit.API.Key.Created.Retrieved-prototype"
    LogTypes = [PantherLogType.CarbonBlack_Audit]
    Description = "Detects when a user creates a new API key or retrieves an existing key."
    DisplayName = "Carbon Black API Key Created or Retrieved"
    Severity = PantherSeverity.Medium
    Tags = ["Persistence", "Create Account"]
    Reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    Reference = "https://docs.vmware.com/en/VMware-Carbon-Black-Cloud/services/carbon-black-cloud-user-guide/GUID-F3816FB5-969F-4113-80FC-03981C65F969.html"
    Tests = carbon_black_audit_api_key_created_retrieved_tests
    PATTERNS = (
        " retrieved secret for API ID ",
        "Added API ID ",
        "Regenerated API key for API ID ",
        "Updated API ID ",
    )

    def rule(self, event):
        desc = event.get("description", "")
        return any((pattern in desc for pattern in self.PATTERNS))

    def title(self, event):
        user = event.get("loginName", "<NO_USERNAME_FOUND>")
        ip_addr = event.get("clientIp", "<NO_IP_FOUND>")
        desc = event.get("description", "<NO_DESCRIPTION_FOUND>")
        return f"{user} [{ip_addr}] {desc}"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context
from pypanther.log_types import PantherLogType

snyk_system_external_access_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Snyk External Access Allowed By External Parties - Enabled",
        ExpectedResult=True,
        Log={
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "event": "group.request_access_settings.edit",
            "content": {"after": {"isEnabled": True}, "before": {}},
            "created": "2023-03-03T19:52:01.628Z",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    PantherRuleTest(
        Name="Snyk External Access Allowed By External Parties - Disabled",
        ExpectedResult=True,
        Log={
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "event": "group.request_access_settings.edit",
            "content": {"after": {}, "before": {"isEnabled": True}},
            "created": "2023-03-03T20:52:01.628Z",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    PantherRuleTest(
        Name="Snyk Group SSO Membership sync",
        ExpectedResult=False,
        Log={
            "content": {
                "addAsOrgAdmin": [],
                "addAsOrgCollaborator": ["group.name"],
                "addAsOrgCustomRole": [],
                "addAsOrgRestrictedCollaborator": [],
                "removedOrgMemberships": [],
                "userPublicId": "05555555-3333-4ddd-8ccc-755555555555",
            },
            "created": "2023-03-15 13:13:13.133",
            "event": "group.sso.membership.sync",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
        },
    ),
]


class SnykSystemExternalAccess(PantherRule):
    RuleID = "Snyk.System.ExternalAccess-prototype"
    DisplayName = "Snyk System External Access Settings Changed"
    LogTypes = [PantherLogType.Snyk_GroupAudit, PantherLogType.Snyk_OrgAudit]
    Tags = ["Snyk"]
    Severity = PantherSeverity.High
    Description = (
        "Detects when Snyk Settings that control access for external parties have been changed.\n"
    )
    Runbook = "This action in the Snyk Audit logs indicate that the setting for allowing external parties to request access to your Snyk installation have changed.\n"
    Reference = (
        "https://docs.snyk.io/snyk-admin/manage-users-and-permissions/organization-access-requests"
    )
    SummaryAttributes = ["event"]
    Tests = snyk_system_external_access_tests
    ACTIONS = ["group.request_access_settings.edit", "org.request_access_settings.edit"]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        current_setting = deep_get(event, "content", "after", "isEnabled", default=False)
        action = deep_get(event, "event", default="<NO_EVENT>")
        if "." in action:
            action = action.split(".")[0].title()
        return f"Snyk: [{action}] External Access settings have been modified to PermitExternalUsers:[{current_setting}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        a_c = snyk_alert_context(event)
        current_setting = deep_get(event, "content", "after", "isEnabled", default=False)
        a_c["current_setting"] = current_setting
        return a_c

    def dedup(self, event):
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}"

    def severity(self, event):
        current_setting = deep_get(event, "content", "after", "isEnabled", default=False)
        if current_setting:
            return "HIGH"
        return "INFO"



from typing import List

from panther_detection_helpers.caching import get_string_set, put_string_set

from pypanther.base import PantherRule, PantherRuleMock, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

git_hub_repo_initial_access_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="GitHub - Initial Access",
        ExpectedResult=True,
        Mocks=[
            PantherRuleMock(ObjectName="get_string_set", ReturnValue=""),
            PantherRuleMock(ObjectName="put_string_set", ReturnValue=""),
        ],
        Log={
            "@timestamp": 1623971719091,
            "business": "",
            "org": "my-org",
            "repo": "my-org/my-repo",
            "action": "git.push",
            "p_log_type": "GitHub.Audit",
            "protocol_name": "ssh",
            "repository": "my-org/my-repo",
            "repository_public": False,
            "actor": "cat",
            "user": "",
        },
    ),
    PantherRuleTest(
        Name="GitHub - Repeated Access",
        ExpectedResult=False,
        Mocks=[PantherRuleMock(ObjectName="get_string_set", ReturnValue='"cat":"my-repo"\n')],
        Log={
            "@timestamp": 1623971719091,
            "business": "",
            "org": "my-org",
            "repo": "my-org/my-repo",
            "action": "git.push",
            "p_log_type": "GitHub.Audit",
            "protocol_name": "ssh",
            "repository": "my-org/my-repo",
            "repository_public": False,
            "actor": "cat",
            "user": "",
        },
    ),
    PantherRuleTest(
        Name="GitHub - Initial Access Public Repo",
        ExpectedResult=False,
        Mocks=[
            PantherRuleMock(ObjectName="get_string_set", ReturnValue=""),
            PantherRuleMock(ObjectName="put_string_set", ReturnValue=""),
        ],
        Log={
            "@timestamp": 1623971719091,
            "business": "",
            "org": "my-org",
            "repo": "my-org/my-repo",
            "action": "git.push",
            "p_log_type": "GitHub.Audit",
            "protocol_name": "ssh",
            "repository": "my-org/my-repo",
            "repository_public": True,
            "actor": "cat",
            "user": "",
        },
    ),
    PantherRuleTest(
        Name="GitHub - Clone without Actor",
        ExpectedResult=False,
        Log={
            "@timestamp": 1623971719091,
            "business": "",
            "org": "my-org",
            "repo": "my-org/my-repo",
            "action": "git.push",
            "p_log_type": "GitHub.Audit",
            "protocol_name": "ssh",
            "repository": "my-org/my-repo",
            "repository_public": False,
            "actor": "",
            "user": "",
        },
    ),
]


class GitHubRepoInitialAccess(PantherRule):
    RuleID = "GitHub.Repo.InitialAccess-prototype"
    DisplayName = "GitHub User Initial Access to Private Repo"
    LogTypes = [PantherLogType.GitHub_Audit]
    Tags = ["GitHub"]
    Reference = "https://docs.github.com/en/organizations/managing-user-access-to-your-organizations-repositories/managing-repository-roles/managing-an-individuals-access-to-an-organization-repository"
    Severity = PantherSeverity.Info
    Description = "Detects when a user initially accesses a private organization repository."
    Tests = git_hub_repo_initial_access_tests
    CODE_ACCESS_ACTIONS = ["git.clone", "git.push", "git.fetch"]

    def rule(self, event):
        # if the actor field is empty, short circuit the rule
        if not event.udm("actor_user"):
            return False
        if event.get("action") in self.CODE_ACCESS_ACTIONS and (not event.get("repository_public")):
            # Compute unique entry for this user + repo
            key = self.get_key(event)
            previous_access = get_string_set(key)
            if not previous_access:
                put_string_set(key, key)
                return True
        return False

    def title(self, event):
        return f"A user [{event.udm('actor_user')}] accessed a private repository [{event.get('repo', '<UNKNOWN_REPO>')}] for the first time."

    def get_key(self, event):
        return __name__ + ":" + str(event.udm("actor_user")) + ":" + str(event.get("repo"))



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get, okta_alert_context
from pypanther.log_types import PantherLogType

okta_phishing_attempt_blocked_fast_pass_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Other Event",
        ExpectedResult=False,
        Log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                }
            },
            "displaymessage": "Authentication of user via MFA",
            "eventtype": "user.authentication.auth_via_mfa",
            "legacyeventtype": "core.user.factor.attempt_fail",
            "outcome": {"reason": "INVALID_CREDENTIALS", "result": "FAILURE"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    }
                ]
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                }
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    PantherRuleTest(
        Name="FastPass Phishing Block Event",
        ExpectedResult=True,
        Log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                }
            },
            "displaymessage": "Authentication of user via MFA",
            "eventtype": "user.authentication.auth_via_mfa",
            "legacyeventtype": "core.user.factor.attempt_fail",
            "outcome": {"reason": "FastPass declined phishing attempt", "result": "FAILURE"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    }
                ]
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                }
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
]


class OktaPhishingAttemptBlockedFastPass(PantherRule):
    RuleID = "Okta.Phishing.Attempt.Blocked.FastPass-prototype"
    DisplayName = "Okta AiTM Phishing Attempt Blocked by FastPass"
    LogTypes = [PantherLogType.Okta_SystemLog]
    Reports = {"MITRE ATT&CK": ["TA0001:T1566", "TA0006:T1556", "TA0003:T1078.004"]}
    Severity = PantherSeverity.High
    Description = (
        "Okta FastPass detected a user targeted by attackers wielding real-time (AiTM) proxies.\n"
    )
    Runbook = "Protect sign-in flows by enforcing phishing-resistant authentication with Okta FastPass and FIDO2 WebAuthn.\n"
    Reference = "https://sec.okta.com/fastpassphishingdetection\n"
    DedupPeriodMinutes = 30
    Tests = okta_phishing_attempt_blocked_fast_pass_tests

    def rule(self, event):
        return (
            event.get("eventType") == "user.authentication.auth_via_mfa"
            and deep_get(event, "outcome", "result") == "FAILURE"
            and (deep_get(event, "outcome", "reason") == "FastPass declined phishing attempt")
        )

    def title(self, event):
        return f"{deep_get(event, 'actor', 'displayName', default='<displayName-not-found>')} <{deep_get(event, 'actor', 'alternateId', default='alternateId-not-found')}> FastPass declined phishing attempt"

    def alert_context(self, event):
        return okta_alert_context(event)
